@@ -5,35 +5,46 @@
 
 #include <config.h>
 
+#include <limits.h>  // for SSIZE_MAX
+#include <stddef.h>  // for size_t
 #include <stdint.h>
 #include <string.h>  // for strcmp, memchr, strlen
 
 #include <algorithm>
-#include <vector>
+#include <iosfwd>    // for nullptr_t
+#include <iterator>  // for distance
+#include <memory>    // for unique_ptr
+#include <string>    // for u16string
 
 #include <gio/gio.h>
-#include <girepository.h>
 #include <glib-object.h>
 #include <glib.h>
 
 #include <js/ArrayBuffer.h>
 #include <js/CallArgs.h>
 #include <js/CharacterEncoding.h>
+#include <js/ErrorReport.h>  // for JS_ReportOutOfMemory
+#include <js/Exception.h>    // for JS_ClearPendingException, JS_...
 #include <js/GCAPI.h>  // for AutoCheckCannotGC
+#include <js/PropertyAndElement.h>
 #include <js/PropertySpec.h>
 #include <js/RootingAPI.h>
+#include <js/String.h>
 #include <js/TypeDecls.h>
-#include <js/Utility.h>   // for UniqueChars
-#include <jsapi.h>        // for JS_DefineFunctionById, JS_DefineFun...
-#include <jsfriendapi.h>  // for JS_NewUint8ArrayWithBuffer, GetUint...
+#include <js/Utility.h>  // for UniqueChars
+#include <js/Value.h>
+#include <js/experimental/TypedData.h>
+#include <jsapi.h>        // for JS_NewPlainObject, JS_InstanceOf
+#include <jsfriendapi.h>  // for ProtoKeyToClass
+#include <jspubtd.h>      // for JSProto_TypeError, JSProto_InternalError
+#include <mozilla/Maybe.h>
+#include <mozilla/Span.h>
+#include <mozilla/Tuple.h>
 #include <mozilla/Unused.h>
 
-#include "gi/boxed.h"
-#include "cjs/atoms.h"
-#include "cjs/context-private.h"
-#include "cjs/deprecation.h"
 #include "cjs/jsapi-util-args.h"
 #include "cjs/jsapi-util.h"
+#include "cjs/macros.h"
 #include "cjs/text-encoding.h"
 
 // Callback to use with JS::NewExternalArrayBuffer()
@@ -112,13 +123,12 @@ static JSString* gjs_lossy_decode_from_uint8array_slow(
 
     do {
         // Create a buffer to convert into.
-        std::vector<char> buffer(buffer_size);
+        std::unique_ptr<char[]> buffer = std::make_unique<char[]>(buffer_size);
         size_t bytes_written = 0, bytes_read = 0;
 
         g_converter_convert(G_CONVERTER(converter.get()), input, input_len,
-                            buffer.data(), buffer.size(),
-                            G_CONVERTER_INPUT_AT_END, &bytes_read,
-                            &bytes_written, &error);
+                            buffer.get(), buffer_size, G_CONVERTER_INPUT_AT_END,
+                            &bytes_read, &bytes_written, &error);
 
         // If bytes were read, adjust input.
         if (bytes_read > 0) {
@@ -129,7 +139,7 @@ static JSString* gjs_lossy_decode_from_uint8array_slow(
         // If bytes were written append the buffer contents to our string
         // accumulator
         if (bytes_written > 0) {
-            char16_t* utf16_buffer = reinterpret_cast<char16_t*>(buffer.data());
+            char16_t* utf16_buffer = reinterpret_cast<char16_t*>(buffer.get());
             // std::u16string uses exactly 2 bytes for every character.
             output_str.append(utf16_buffer, bytes_written / 2);
         } else if (error) {
@@ -190,7 +200,7 @@ static JSString* gjs_lossy_decode_from_uint8array_slow(
 GJS_JSAPI_RETURN_CONVENTION
 static JSString* gjs_decode_from_uint8array_slow(JSContext* cx,
                                                  const uint8_t* input,
-                                                 uint32_t input_len,
+                                                 size_t input_len,
                                                  const char* encoding,
                                                  bool fatal) {
     // If the decoding is not fatal we use the lossy decoder.
@@ -198,6 +208,12 @@ static JSString* gjs_decode_from_uint8array_slow(JSContext* cx,
         return gjs_lossy_decode_from_uint8array_slow(cx, input, input_len,
                                                      encoding);
 
+    // g_convert only handles up to SSIZE_MAX bytes, but we may have SIZE_MAX
+    if (G_UNLIKELY(input_len > SSIZE_MAX)) {
+        gjs_throw(cx, "Array too big to decode: %zu bytes", input_len);
+        return nullptr;
+    }
+
     size_t bytes_written, bytes_read;
     GError* error = nullptr;
 
@@ -235,14 +251,13 @@ static JSString* gjs_decode_from_uint8array_slow(JSContext* cx,
 }
 
 // Finds the length of a given data array, stopping at the first 0 byte.
-template <class T, class L>
-[[nodiscard]] static L zero_terminated_length(const T* data, L len) {
+template <class T>
+[[nodiscard]] static size_t zero_terminated_length(const T* data, size_t len) {
     if (!data || len == 0)
         return 0;
 
     const T* start = data;
-    auto* found = static_cast<const T*>(
-        std::memchr(start, '\0', static_cast<size_t>(len)));
+    auto* found = static_cast<const T*>(std::memchr(start, '\0', len));
 
     // If a null byte was not found, return the passed length.
     if (!found)
@@ -264,8 +279,7 @@ JSString* gjs_decode_from_uint8array(JSContext* cx, JS::HandleObject byte_array,
     }
 
     uint8_t* data;
-    // len should be size_t but SpiderMonkey defines it differently in mozjs78
-    uint32_t len;
+    size_t len;
     bool is_shared_memory;
     js::GetUint8ArrayLengthAndData(byte_array, &len, &is_shared_memory, &data);
 
@@ -321,7 +335,7 @@ JSString* gjs_decode_from_uint8array(JSContext* cx, JS::HandleObject byte_array,
     }
 
     uint8_t* current_data;
-    uint32_t current_len;
+    size_t current_len;
     bool ignore_val;
 
     // If a garbage collection occurs between when we call
@@ -408,7 +422,7 @@ JSObject* gjs_encode_to_uint8array(JSContext* cx, JS::HandleString str,
             JS::AutoCheckCannotGC nogc;
             size_t len;
 
-            if (JS_StringHasLatin1Chars(str)) {
+            if (JS::StringHasLatin1Chars(str)) {
                 const JS::Latin1Char* chars =
                     JS_GetLatin1StringCharsAndLength(cx, nogc, str, &len);
                 if (!chars)
